package openmind;
//import OMCSNet.*;
import java.util.*;
import java.io.*;
/**
 * Austin created Omcs.java at 2:13:05 PM on Nov 16, 2002
 */
public class Omcs {
	FileIO fileIO = new FileIO();
	
	Hashtable profNodes = new Hashtable();
	Hashtable relNodes = new Hashtable();
	Hashtable ageNodes = new Hashtable();
	Hashtable raceNodes = new Hashtable();
	
	Hashtable profEdges = new Hashtable();
	Hashtable relEdges = new Hashtable();
	Hashtable ageEdges = new Hashtable();
	Hashtable raceEdges = new Hashtable();
	
	Vector profSyns = new Vector();
	Vector relSyns = new Vector();
	Vector ageSyns = new Vector();
	Vector raceSyns = new Vector(); 
	
	String filename = "semanticnet.txt";
	//String filename = "test.txt";
	
	public Omcs(){
		
		profSyns = fileIO.readFileToVector("synonyms/prof_synonyms.txt");
		relSyns = fileIO.readFileToVector("synonyms/rel_synonyms.txt");
		ageSyns = fileIO.readFileToVector("synonyms/age_synonyms.txt");
		raceSyns = fileIO.readFileToVector("synonyms/race_synonyms.txt");
		
		System.out.println("Loading semantic net...");
		createNet();
	
	}
	
	public Omcs(boolean empty){
	}
	
	public AAANode getNode(String nodename){
		if(profNodes.containsKey(nodename)) return (AAANode) profNodes.get(nodename);
		if(relNodes.containsKey(nodename)) return (AAANode) relNodes.get(nodename);
		if(ageNodes.containsKey(nodename)) return (AAANode) ageNodes.get(nodename);
		if(raceNodes.containsKey(nodename)) return (AAANode) raceNodes.get(nodename);
		return null;
	}
	
	public AAANode[] getRelation(String relation){
		AAANode[] nodes = new AAANode[2];
		if(profEdges.containsKey(relation)) return (AAANode[]) profEdges.get(relation);
		if(relEdges.containsKey(relation)) return (AAANode[]) relEdges.get(relation);
		if(ageEdges.containsKey(relation)) return (AAANode[]) ageEdges.get(relation);
		if(raceEdges.containsKey(relation)) return (AAANode[]) raceEdges.get(relation);
		return null;
	}
	
	
	public void addEdge(String nodeName,String edgeName,Edge e,int category){
		String[] nodes = new String[2];
		if (category==0) {//profession
			Vector edges = new Vector();
			if(profEdges.containsKey(edgeName)) edges = (Vector) profEdges.get(edgeName);
			nodes[0] = nodeName;
			nodes[1] = e.Target;
			edges.add(nodes);
			profEdges.remove(edgeName);
			profEdges.put(edgeName,edges);
		} 
		else if (category==1) {//religion
			Vector edges = new Vector();
			if(relEdges.containsKey(edgeName)) edges = (Vector) relEdges.get(edgeName);
			nodes[0] = nodeName;
			nodes[1] = e.Target;
			edges.add(nodes);
			profEdges.remove(edgeName);
			relEdges.put(edgeName,edges);
		} 
		else if (category==2) {//age
			Vector edges = new Vector();
			if(ageEdges.containsKey(edgeName)) edges = (Vector) ageEdges.get(edgeName);
			nodes[0] = nodeName;
			nodes[1] = e.Target;
			edges.add(nodes);
			profEdges.remove(edgeName);
			ageEdges.put(edgeName,edges);
		} 
		else if (category==3) {//race
			Vector edges = new Vector();
			if(raceEdges.containsKey(edgeName)) edges = (Vector) raceEdges.get(edgeName);
			nodes[0] = nodeName;
			nodes[1] = e.Target;
			edges.add(nodes);
			profEdges.remove(edgeName);
			raceEdges.put(edgeName,edges);
		}
	}
	 
	
	public void createNet(){
		profNodes = new Hashtable();
		relNodes = new Hashtable();
		ageNodes = new Hashtable();
		raceNodes = new Hashtable();
		
		profEdges = new Hashtable();
		relEdges = new Hashtable();
		ageEdges = new Hashtable();
		raceEdges = new Hashtable();
		
		StreamTokenizer st = fileIO.readStreamFile(filename);
		try{
			while (st.ttype!=StreamTokenizer.TT_EOF){
				st.nextToken();
				String next = st.sval;
				if(next.startsWith("NODE:")){
					String nodeName;
					nodeName = next.substring(6);
					AAANode node = new AAANode(nodeName);
					st.nextToken();
					while(!next.equals("*********************************")){
						st.nextToken();
						next = st.sval;
						String pred = next.substring(6);
						//System.out.println(pred);
						
						st.nextToken();
						next = st.sval;
						String tar = next.substring(8);
						//System.out.println(tar);
						
						st.nextToken();
						next = st.sval;
						String sen = next.substring(10);
						//System.out.println(sen);
						
						st.nextToken();
						next = st.sval;
						String dir = next.substring(11);
						//System.out.println(dir);
						
						st.nextToken();
						next = st.sval;
						String weight = next.substring(8);
						//System.out.println(weight);
						
						st.nextToken();
						next = st.sval;
						Edge e = new Edge(pred,tar,sen,dir,weight);
						node.addEdge(e);
						if(profSyns.contains(nodeName)) addEdge(nodeName,pred,e,0);
						else if(relSyns.contains(nodeName)) addEdge(nodeName,pred,e,1);
						else if(ageSyns.contains(nodeName)) addEdge(nodeName,pred,e,2);
						else if(raceSyns.contains(nodeName)) addEdge(nodeName,pred,e,3);
					}
					//System.out.println(nodeName);
					if(profSyns.contains(nodeName)) profNodes.put(nodeName, node);
					else if(relSyns.contains(nodeName)) relNodes.put(nodeName, node);
					else if(ageSyns.contains(nodeName)) ageNodes.put(nodeName, node);
					else if(raceSyns.contains(nodeName)) raceNodes.put(nodeName, node);
				}
				//System.out.println(".");
			}
		}
		catch(Exception e){}
	}
	
	public void filter(){
		for(int i=0;i<profSyns.size();i++){
			AAANode node = getNode((String) profSyns.get(i));
			if(node!=null){
				Vector edges = node.edges;
				for (int j=0;j<edges.size();j++){
					Edge edge = (Edge) edges.get(j);
					String relation = edge.Pred;
					Vector nodePairs = new Vector();
					if(profEdges.containsKey(relation)){
						nodePairs = (Vector) profEdges.get(relation);
					}
					for (int k=0;k<edges.size();k++){
						
					}
				}
			}
				
		}
	}
	
	public static void main(String[] args) {
		Omcs omcs = new Omcs();
		AAANode testNode = omcs.getNode("musician");
		System.out.println(testNode);
		
	}
}
