package openmind;
import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import touchgraph.graphlayout.*;

/**
 * Austin created VisAAANet.java at 6:57:33 PM on Nov 16, 2002
 */
public class VisAAANet extends JPanel implements ActionListener{
	VisualNet pVisual;
	JPanel pInput;
	JButton bCreate, bClear;
	Omcs omcs;
	String[] aRelations;
	JCheckBox[] cbRelations;
	Vector vRelations;
	Color[] nodeColours;
	Color[] edgeColours;
	Vector vConcepts = new Vector();

	public VisAAANet(){

		//colours
		nodeColours = new Color[4];
		Random r = new Random();
		for(int i=0;i<4;i++){
			nodeColours[i] = new Color(r.nextInt(255),r.nextInt(255),r.nextInt(255));
		}	
		
		edgeColours = new Color[16];
		for(int i=0;i<16;i++){
			edgeColours[i] = new Color(r.nextInt(255),r.nextInt(255),r.nextInt(255));
		}
		//String[] temp = {"hasAbility","hasCollocate","hasConsequence","hasEffect","hasFirstStep","hasLocation","hasMotivation","hasPart","hasProperty","hasRequirement","hasResultingWant","hasStep","hasUse","hasUseLocation","hasWant","isA"};
		String[] temp = {"hasAbility","hasConsequence","hasEffect","hasFirstStep","hasMotivation","hasPart","hasProperty","hasRequirement","hasResultingWant","hasStep","hasUse","hasUseLocation","hasWant", "isA"};
		aRelations = temp;
		vRelations = new Vector();
		for(int i=0;i<temp.length;i++){
			vRelations.add(temp[i]);
		}

		//pVisual
		pVisual = new VisualNet(); //extends JPanel
		omcs = new Omcs(true);
		pVisual.changeDefaultLength(100);	    	
		pVisual.setBorder(BorderFactory.createCompoundBorder(
							     BorderFactory.createTitledBorder("AAA Visualisation"),
							     BorderFactory.createEmptyBorder(5,5,5,5)));
		
		setLayout(new BorderLayout());
		add(pVisual, BorderLayout.CENTER);
	}
	
	public VisAAANet(boolean button){
		//colours
		nodeColours = new Color[4];
		Random r = new Random();
		for(int i=0;i<4;i++){
			nodeColours[i] = new Color(r.nextInt(255),r.nextInt(255),r.nextInt(255));
		}	
		
		edgeColours = new Color[16];
		for(int i=0;i<16;i++){
			edgeColours[i] = new Color(r.nextInt(255),r.nextInt(255),r.nextInt(255));
		}
		

		//pInput
		bCreate = new JButton("VISUALIZE BABY!!");
		bCreate.addActionListener(this);
		bClear = new JButton("Clear");
		bClear.addActionListener(this);
		
		pInput = new JPanel();
		pInput.setLayout(new GridLayout(5, 4, 1, 1));
		pInput.setBackground(Color.white);		
		pInput.setBorder(BorderFactory.createCompoundBorder(
							    BorderFactory.createTitledBorder("Query Parameters"),
							    BorderFactory.createEmptyBorder(5,5,5,5)));
		
		cbRelations = new JCheckBox[16];
		String[] temp = {"hasAbility","hasCollocate","hasConsequence","hasEffect","hasFirstStep","hasLocation","hasMotivation","hasPart","hasProperty","hasRequirement","hasResultingWant","hasStep","hasUse","hasUseLocation","hasWant","isA"};
		aRelations = temp;
		
		for(int i=0;i<cbRelations.length;i++){
			cbRelations[i] = new JCheckBox(aRelations[i]);
			JPanel pTemp = new JPanel();
			pTemp.setBackground(edgeColours[i]);
			pTemp.add(cbRelations[i]);
			pInput.add(pTemp);
		}
		
		pInput.add(bCreate);
		pInput.add(bClear);
		
		//pVisual
		pVisual = new VisualNet(); //extends JPanel
		omcs = new Omcs();
		pVisual.changeDefaultLength(100);	    	
		pVisual.setBorder(BorderFactory.createCompoundBorder(
							     BorderFactory.createTitledBorder("AAA Visualisation"),
							     BorderFactory.createEmptyBorder(5,5,5,5)));
		
		
		setLayout(new BorderLayout());
		add(pInput, BorderLayout.NORTH);
		add(pVisual, BorderLayout.CENTER);
		
	}

  	public void actionPerformed(ActionEvent ae){
		if(ae.getSource() == bCreate){
			vConcepts = new Vector();
			Thread tOutput = new Thread(){
				public void run(){
					vRelations = new Vector();
					for(int i=0;i<cbRelations.length;i++){
						if(cbRelations[i].isSelected()){
							vRelations.add(aRelations[i]);
						}
					}
			  		//addOmcsNodes();
					populateAAANodes();
				}		
		   	};
		tOutput.start();
	    }
	    else if(ae.getSource() == bClear){
	    	pVisual.clearAll();
	    }
  	}
  	
  	public Vector addAAANodes(){//add nodes in the AAA mode
  		//add all nodes, but only show the "YOU" node and its attributes, hide everything else
  		addAttributeNodes(omcs.profNodes,0);
  		addAttributeNodes(omcs.relNodes,1);
  		addAttributeNodes(omcs.ageNodes,2);
  		addAttributeNodes(omcs.raceNodes,3);
  		return vConcepts;
  	}
  	
  	private void addAttributeNodes(Hashtable hsNodes, int type){
  		Enumeration enu = hsNodes.keys();
		while(enu.hasMoreElements()){
			AAANode node = (AAANode) hsNodes.get(enu.nextElement());
			String nodeName = node.name;
			String temp = nodeName.substring(0,1).toUpperCase();
			nodeName = temp+nodeName.substring(1,nodeName.length());
			pVisual.addColouredNode("YOU", nodeName, Color.BLACK, Color.RED, Color.GRAY);
			Vector edges = node.edges;
			for(int i=0;i<edges.size();i++){
				Edge edge = (Edge) edges.get(i);
				for(int j=0;j<vRelations.size();j++){
					String relation = (String) vRelations.get(j);
					if(edge.Pred.startsWith(relation)) {
						int colour = 0;
						for(int k=0;k<aRelations.length;k++){
							if(aRelations[k].equals(relation)) colour = k;
						}
						vConcepts.add(edge.Target);
						addHiddenNode(nodeName,edge.Target,type,colour);
					}
				}
			}

		}
	}
  	
  	public Vector addOmcsNodes (){
  		addNodes(omcs.profNodes,0);
	    addNodes(omcs.relNodes,1);
	    addNodes(omcs.ageNodes,2);
	    addNodes(omcs.raceNodes,3);
	    return vConcepts;
  	}
  	
    private void addNodes(Hashtable hsNodes, int type){
    	Enumeration enu = hsNodes.keys();
    	while(enu.hasMoreElements()){
	    	AAANode node = (AAANode) hsNodes.get(enu.nextElement());
	    	String nodeName = node.name;
    		String temp = nodeName.substring(0,1).toUpperCase();
    		nodeName = temp+nodeName.substring(1,nodeName.length());
	    	pVisual.addColouredNode("YOU", nodeName, Color.BLACK, Color.RED, Color.GRAY);
    		Vector edges = node.edges;
	    	for(int i=0;i<edges.size();i++){
	    		Edge edge = (Edge) edges.get(i);
	    		for(int j=0;j<vRelations.size();j++){
	    			String relation = (String) vRelations.get(j);
	    			if(edge.Pred.startsWith(relation)) {
	    				int colour = 0;
	    				for(int k=0;k<aRelations.length;k++){
	    					if(aRelations[k].equals(relation)) colour = k;
	    				}
	    				vConcepts.add(edge.Target);
	    				//addNode(nodeName,edge.Target,type,colour);
	    			}
	    		}
	    	}
    	}
    }
    
    public void populateAAANodes(){
		addAAANodes(omcs.profNodes,0);
		addAAANodes(omcs.relNodes,1);
		addAAANodes(omcs.ageNodes,2);
		addAAANodes(omcs.raceNodes,3);
	}
    
	private void addAAANodes(Hashtable hsNodes, int type){
		Enumeration enu = hsNodes.keys();
		while(enu.hasMoreElements()){
			AAANode node = (AAANode) hsNodes.get(enu.nextElement());
			String nodeName = node.name;
			Vector edges = node.edges;
			for(int i=0;i<edges.size();i++){
				Edge edge = (Edge) edges.get(i);
				for(int j=0;j<vRelations.size();j++){
					String relation = (String) vRelations.get(j);
					if(edge.Pred.startsWith(relation)) {
						int colour = 0;
						for(int k=0;k<aRelations.length;k++){
							if(aRelations[k].equals(relation)) colour = k;
						}
						addNode(nodeName,edge.Target,type,colour);
					}
				}
			}
		}
	}

	public void addHiddenNode(String parent, String child, int nodeColour, int edgeColour){
		pVisual.addHiddenNode(parent, child, nodeColours[nodeColour], edgeColours[edgeColour]);
	}
    
    public void addNode(String parent, String child, int nodeColour, int edgeColour){
		pVisual.addColouredNode(parent, child, nodeColours[nodeColour], edgeColours[edgeColour]);
    }
    
	public void addProductNode(String parent, String child, Color childColour, Color edgeColour){
		pVisual.addColouredNode(parent, child, Color.BLACK, childColour, edgeColour);
	}
	
	public void addConceptNode(String parent, String child, Color childColour, Color edgeColour){
		pVisual.addColouredNode(parent, child, Color.RED, childColour, edgeColour);
	}

	
	
	public void clearAll(){
		pVisual.clearAll();
		vConcepts = new Vector();
	}
}
