package openmind;
import java.util.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import touchgraph.graphlayout.*;

public class VisRecommend extends JPanel implements ActionListener{
    JPanel pInput, pEmpty, pLevels;
    VisualNet pVisual;
    JLabel lNickname, lRecommend, lStatus, lLevels;
    JTextField tfNickname, tfRecommend, tfLevels;
    Color[] colours;
    JButton bVisualize;
    String strNickname, strRecommend, strProfession, strReligion, strRace,  strAge, sResults;
    FileWriter fwProfileFile;
    StringReader srProfileInfo;
    boolean bFlag = false;
    int intLevels, intMaxChildren;
    Connection con;
    ArrayList alFilePairs;
    FileIO fileIO;
    
    public VisRecommend(){
    	intMaxChildren = 10;	//limits the number of child nodes
		pVisual = new VisualNet(); //extends JPanel
		pInput = new JPanel();
		pLevels = new JPanel();
		lNickname = new JLabel("Nickname:");
		lRecommend = new JLabel("Subject to Recommend:");
		lLevels = new JLabel("# levels to expand:");
		lStatus = new JLabel("");
		tfNickname = new JTextField();
		tfRecommend = new JTextField();
		tfLevels = new JTextField("1");
		bVisualize = new JButton("Visualize");
	
		pInput.setBackground(Color.white);
		pLevels.setBackground(Color.white);
		
		bVisualize.addActionListener(this);
		
		pInput.setLayout(new GridLayout(3, 3, 1, 1));
		
		pInput.add(lNickname);
		pInput.add(tfNickname);
		pEmpty = new JPanel();
		pEmpty.setBackground(Color.white);
		pInput.add(pEmpty);
		
		pInput.add(lRecommend);
		pInput.add(tfRecommend);
		pInput.add(bVisualize);
		
		pInput.add(lLevels);
		pInput.add(tfLevels);
		pInput.add(lStatus);
		
		con = new Connection("http://commonsense.media.mit.edu/cgi-bin/get.cgi?");
		fileIO = new FileIO();
		
		pInput.setBorder(BorderFactory.createCompoundBorder(
								    BorderFactory.createTitledBorder("Query Parameters"),
								    BorderFactory.createEmptyBorder(5,5,5,5)));
		
		pVisual.setBorder(BorderFactory.createCompoundBorder(
								     BorderFactory.createTitledBorder("Recommendation Visualisation"),
								     BorderFactory.createEmptyBorder(5,5,5,5)));
		
		setLayout(new BorderLayout());
		//setPreferredSize(new Dimension(100, 100));
		add(pInput, BorderLayout.NORTH);
		add(pVisual, BorderLayout.CENTER);
    }

    public void actionPerformed(ActionEvent ae){
		strNickname = tfNickname.getText();
		strRecommend = tfRecommend.getText();
		try{intLevels = (new Integer(tfLevels.getText())).intValue();}
	    catch (Exception e){intLevels = 1;lStatus.setText("# levels = 1");}
		if(strNickname == null || strNickname.equals("")) lStatus.setText("Error: username missing");
		if(strRecommend == null || strRecommend.equals("")) lStatus.setText("Error: topic missing");
		
		else{
			lStatus.setText("Loading profile of " + strNickname + "...");
		    /*  New thread here to do the connection call so that it doesn't hang the whole GUI */
		    Thread tOutput = new Thread(){
			    public void run(){
					try{
				    	//Here we read in the profile
    					String sFile = fileIO.readFile(strNickname + ".txt");
				    				
				    	//populate graph with user profile nodes
				    	lStatus.setText("Building attribute graph of " + strNickname + "...");
				    	populateProfileNodes(sFile);
				    
				    	//generate random colours for all levels
				    	lStatus.setText("Generating colours" + "...");
				    	colours = new Color[intLevels];
					    Random r = new Random();
					    for(int i=0;i<intLevels;i++){
							colours[i] = new Color(r.nextInt(255),r.nextInt(255),r.nextInt(255));
					    }
					    //expand on topic
					    lStatus.setText("Building graph with "+strRecommend+"...");
					    expand(strRecommend, intLevels);
					    lStatus.setText("");
					}
					catch(Exception e){
					    System.out.println(e.toString());
					}
			    }
			};
		    
		    tOutput.start();
		}
    }
       
	public void expand (String parent, int levels){
    	if (levels>0) {
    		String sResults = con.getResults(parent);
		    sResults = fileIO.sort(fileIO.filterAndCount(sResults));
			StringTokenizer st = new StringTokenizer(sResults,"\n");
			int counter = intMaxChildren;
			while(st.hasMoreTokens()&&counter>0){
				counter--;
				String token = st.nextToken();
				Pair pair = new Pair(token); 
				pVisual.addTargetNode(parent, pair.getWord(), colours[levels-1]);
				expand(pair.getWord(), levels-1);
			}
		}
    }
	
	public void populateProfileNodes(String sFile){
		//generate random colors for nodes
		Color[] colors = new Color[2];
		Random r = new Random();
		colors[0] = new Color(r.nextInt(255),r.nextInt(255),r.nextInt(255));
		colors[1] = new Color(r.nextInt(255),r.nextInt(255),r.nextInt(255));
		
		//add OpenMind nodes
		StringTokenizer st = new StringTokenizer(sFile,"\n");
	    String sWord, sValue;
	    int resultType = 0;
	    String parent = "";
	    int max = intMaxChildren;
	    while(st.hasMoreTokens()){
	    	String token = st.nextToken();
	    	try{    		
				if(token.startsWith("---Profession---:")) {
					lStatus.setText("Expanding profession...");
					strProfession=token.substring(token.indexOf(":")+1,token.length());
				    token = st.nextToken();
				    max = intMaxChildren;
				    resultType = 1;
				    parent = strProfession;
				    pVisual.addTargetNode(strNickname,strProfession,colors[0],1);
				}
				else if(token.startsWith("---Religion---:")) {
					lStatus.setText("Expanding religion...");
					strReligion=token.substring(token.indexOf(":")+1,token.length());
				    token = st.nextToken();
				    max = intMaxChildren;
				    resultType = 2;
				    parent = strReligion;
				    pVisual.addTargetNode(strNickname,strReligion,colors[0],2);
				}
				else if(token.startsWith("---Race---:")) {
					lStatus.setText("Expanding race...");
					strRace=token.substring(token.indexOf(":")+1,token.length());
				    token = st.nextToken();
				    max = intMaxChildren;
				    resultType = 3;
				    parent = strRace;
				    pVisual.addTargetNode(strNickname,strRace,colors[0],3);
				}
				else if(token.startsWith("---Age---:")) {
					lStatus.setText("Expanding age...");
					strAge=token.substring(token.indexOf(":")+1,token.length());
				    token = st.nextToken();
				    max = intMaxChildren;
				    resultType = 4;
				    parent = strAge;
				    pVisual.addTargetNode(strNickname,strAge,colors[0],4);
				}
		    }
	    	catch (Exception e){
	    		System.out.println("Exception in adding "+parent+" node: "+e+" child: "+token);
	    	}
	    	if(resultType!=0&&!token.equals("")&&max>0) {
	    		max--;
	    		Pair p = new Pair(token);
				addNode(parent, p.getWord(), resultType, colors[1]);
		    }
	    }
	}
	public void addNode(String parent, String word, int resultType, Color colour){
		switch(resultType){
			case 1: pVisual.addTargetNode(parent, word, colour, 1);break;
			case 2: pVisual.addTargetNode(parent, word, colour, 2);break;
			case 3: pVisual.addTargetNode(parent, word, colour, 3);break;
			case 4: pVisual.addTargetNode(parent, word, colour, 4);break;
		}
    }
}

