package touchgraph.graphlayout;


import  java.awt.*;
import  java.awt.event.*;
import  javax.swing.*;
import  java.util.*;

public class VisualNet extends GLPanel {
	private int defaultLength = 100;
	private Vector hiddenNodes = new Vector();
	
    public void initialize() {
        buildPanel();
        buildLens();
        tgPanel.setLensSet(tgLensSet);
        addUIs();
        setVisible(true);
    }

	private Node addNode(String name) {
		Node node = new Node(name,name);
		try{tgPanel.addNode(node);}
		//catch(Exception e){System.out.println("Exception in adding node: "+e);}//catches NODE_EXISTS exception
		catch(Exception e){}
		return tgPanel.completeEltSet.findNode(name);
    }
    
    private Node addNode(String name, Color colour) {
		Node node = new Node(name,0,colour,name);
		try{tgPanel.addNode(node);}
		//catch(Exception e){System.out.println("Exception in adding node: "+e);}//catches NODE_EXISTS exception
		catch(Exception e){}
		return tgPanel.completeEltSet.findNode(name);
    }
    
    private Node addNode(String name, int shape ,Color colour) {
		Node node = new Node(name,shape,colour,name);
		try{tgPanel.addNode(node);}
		//catch(Exception e){System.out.println("Exception in adding node: "+e);}//catches NODE_EXISTS exception
		catch(Exception e){}
		return tgPanel.completeEltSet.findNode(name);
    }
    
	private Node addHiddenNode(String name, Color colour) {
		hiddenNodes.add(name);
		Node node = new Node(name,0,colour,name);
		node.setVisible(false);
		try{tgPanel.addNode(node);}
		//catch(Exception e){System.out.println("Exception in adding node: "+e);}//catches NODE_EXISTS exception
		catch(Exception e){}
		return tgPanel.completeEltSet.findNode(name);
	}
    
    private void addEdge(Node parent, Node child, Color color) throws TGException{
		if (!parent.equals(child)) {
			parent.setVisible(true);
			child.setVisible(true);
			Edge edge = new Edge(parent,child,defaultLength);
			edge.from.addEdge(edge);
            edge.to.addEdge(edge);
			edge.setColor(color);
			tgPanel.addEdge(edge);
		}
	}
	        
	private void addHiddenEdge(Node parent, Node child, Color color) {
		if (!parent.equals(child)) {
			Edge edge = new Edge(parent,child,defaultLength);
			edge.from.addEdge(edge);
            edge.to.addEdge(edge);
			edge.setColor(color);//set colour to white if you want invisible
			edge.setVisible(false);
			try{tgPanel.addEdge(edge);}
			catch(Exception e){System.out.println("Exception in adding invisible edges: "+e);}
		}
	}
	
	public void changeDefaultLength(int length){
		defaultLength = length;
	}
	
    public void clearAll(){
    	tgPanel.clearAll();
    	tgPanel.updateUI();
    }
    
    public void addSynonymNode(String parent, String child, Color nodeColour){
		try{addHiddenEdge(addNode(parent), addNode(child, nodeColour), Color.orange);}
		catch(Exception e){System.out.println("Exception in adding synonym node: "+e);}
    }
    public void addTermNode(String parent, String child, Color nodeColour){
		try{addEdge(addNode(parent), addNode(child, nodeColour), Color.green);}
		catch(Exception e){System.out.println("Exception in adding term node: "+e);}
    }
    public void addTargetNode(String parent, String child, Color nodeColour){
		try{addEdge(addNode(parent), addNode(child, nodeColour), Color.blue);}
		catch(Exception e){System.out.println("Exception in adding target node: "+e+" parent: "+parent+" child: "+child);}
    }
    public void addTargetNode(String parent, String child, Color nodeColour, int shape){
		try{addEdge(addNode(parent), addNode(child, shape, nodeColour), Color.blue);}
		catch(Exception e){System.out.println("Exception in adding shaped target node: "+e+" parent: "+parent+" child: "+child+" shape: "+shape);}
    }
    public void addSourceNode(String parent, String child, Color nodeColour){
		try{addEdge(addNode(parent, nodeColour), addNode(child), Color.yellow);}
		catch(Exception e){System.out.println("Exception in adding source node: "+e);}
    }
    
    public void addHiddenNode(String parent, String child, Color nodeColour, Color edgeColour){
    	try{addHiddenEdge(addNode(parent), addHiddenNode(child, nodeColour), edgeColour);}
    	catch(Exception e){System.out.println("Exception in adding source node: "+e);}
    }
    
    public void addColouredNode(String parent, String child, Color nodeColour, Color edgeColour){
		try{addEdge(addNode(parent), addNode(child, nodeColour), edgeColour);}
		catch(Exception e){System.out.println("Exception in adding source node: "+e);}
    }
    
	public void addColouredNode(String parent, String child, Color parentColour,Color childColour, Color edgeColour){
			try{addEdge(addNode(parent,parentColour), addNode(child, childColour), edgeColour);}
			catch(Exception e){System.out.println("Exception in adding source node: "+e);}
	}
      
    public static void main(String[] args) {

        JFrame frame;
        frame = new JFrame("Graph Layout");
        VisualNet visualnet = new VisualNet();
        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {System.exit(0);}
        });

        frame.getContentPane().add("Center", visualnet);
        frame.setSize(500,500);
        frame.setVisible(true);
    }

} // end touchgraph.graphlayout.GLPanel
